#!/usr/bin/env python
# -*- coding: utf-8 -*-

#WRITTEN BY DENE BOWDALO

###------------------------------------------------------------------------------------###
###IMPORT MODULES
###------------------------------------------------------------------------------------###

import crossnetwork_processing_modules
import numpy as np
import os
import subprocess
import time

###------------------------------------------------------------------------------------###
###------------------------------------------------------------------------------------###

#PROGRAM THAT HANDLES THE SUBMISSION OF JOBS FOR DESIRED STAGES OF THE GHOST PIPELINE 

###------------------------------------------------------------------------------------###
#SUMMARY OF STAGES:
###------------------------------------------------------------------------------------###
#N1 = Raw network data parse
#N2 = Resolve duplicate stations
#N3 = Parse globally gridded metadata
#N4 = Quality control
#N5 = Process classifications

#M1 = Process static modal metadata .csv

#T1 = Temporally grid by station
#T2 = Group temporally gridded data to monthly NETCDFs
###------------------------------------------------------------------------------------###

#BEGIN PROCESS!

###------------------------------------------------------------------------------------###
#INITIALISATION : 1. Read configuration file for the setup of the processing system
#                 2. Iterate by network, by species
#                 3. Start timing clock for network/species combination
#                 4. Determine which stages of processing are wished to be done - taken from command line arguments 
###------------------------------------------------------------------------------------###

#Read configuration file
from GHOST_configuration import networks, parameters, start_date, end_date, temporal_resolutions, n_processes, stages_to_process, ghost_version

#set number of processes to run in parallel
#1 Master + N Slave processes 
n_processes = n_processes+1

#Iterate through networks
for network in networks:

    #Iterate through parameters
    for parameter in parameters:
        
        #Set process_code as 0 (valid)
        process_code = 0
    
        #Process configuration variables to be read by all callable modules/submodules --> save data in configuration.npz file
        start_year =  start_date[:4]
        end_year =    end_date[:4]
        start_month = start_date[4:]
        end_month =   end_date[4:]
        run_details = {'network':network,'parameter':parameter,'start_date':start_date,'end_date':end_date,'start_year':start_year,'end_year':end_year,'start_month':start_month,'end_month':end_month,'ghost_version':ghost_version}
        parameter_details = crossnetwork_processing_modules.get_parameter_details()[run_details['parameter']]
        network_details = crossnetwork_processing_modules.get_network_details()[run_details['network']]
        np.savez_compressed('GHOST_configuration.npz', run_details=run_details, parameter_details=parameter_details, network_details=network_details)

        #Start timer
        print 'BEGINNING PROCESSING OF STAGES: %s FOR %s:%s'%(stages_to_process,network,parameter)
        t0 = time.time()
        
###------------------------------------------------------------------------------------###
#STAGE N1 : 1. Process raw network files for a species
#           2. Outputs files to /native/component/temporary directory as an .npz per station 
###------------------------------------------------------------------------------------###
    
        if ('N1' in stages_to_process) & (process_code == 0):
            process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'N1', t0)
            
###------------------------------------------------------------------------------------###
#STAGE N2 : 1. Ensure /native/component/temporary/*.npz files are station specific 
###------------------------------------------------------------------------------------###

        if ('N2' in stages_to_process) & (process_code == 0):
            process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'N2', t0)

###------------------------------------------------------------------------------------###
#STAGE N3 : 1.
###------------------------------------------------------------------------------------###

        #if ('N3' in stages_to_process) & (process_code == 0):
        #    process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'N3', t0)

###------------------------------------------------------------------------------------###
#STAGE N4 : 1. Do Quality Control per station 
#           2. Outputs to /native/component/ directory as an .npz per station
###------------------------------------------------------------------------------------###
        
        if ('N4' in stages_to_process) & (process_code == 0):
            process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'N4', t0)

###------------------------------------------------------------------------------------###
#STAGE N5 : 1. Do Classification checks per station
#           2. Rewrites .npz per station to /native/component/ directory
###------------------------------------------------------------------------------------###
      
        if ('N5' in stages_to_process) & (process_code == 0):
            process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'N5', t0)
            
###------------------------------------------------------------------------------------###
#STAGE M1 : 1. Calculate modal metadata per station across entire time record
#           2. Write to /metadata/processed .csv file
###------------------------------------------------------------------------------------###
      
        if ('M1' in stages_to_process) & (process_code == 0):
            process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'M1', t0)

###------------------------------------------------------------------------------------###
        
        if process_code == 0:
        
            #Set process_code as 1 (no data) - will be overwritten afterwards if processing is valid
            process_code = 1
        
            #Iterate through desired temporal resolutions to average data onto
            for temporal_resolution in temporal_resolutions:
            
                #Set temporal process code as 0 (valid)
                temporal_process_code = 0
            
                #Add temporal_resolution, temporal_resolution_datetime_frequency, temporal_resolution_path and descriptive_temporal_resolution to run_details --> save data to configuration.npz file
                if temporal_resolution == 'H':
                    temporal_resolution_datetime_frequency = 'H'
                    temporal_resolution_path = 'hourly'
                    descriptive_temporal_resolution = 'hours'
                elif temporal_resolution == 'D':
                    temporal_resolution_datetime_frequency = 'D'
                    temporal_resolution_path = 'daily'
                    descriptive_temporal_resolution = 'days'
                elif temporal_resolution == 'M':
                    temporal_resolution_datetime_frequency = 'MS'
                    temporal_resolution_path = 'monthly'
                    descriptive_temporal_resolution = 'months'
                run_details['temporal_resolution'] = temporal_resolution
                run_details['temporal_resolution_datetime_frequency'] = temporal_resolution_datetime_frequency
                run_details['temporal_resolution_path'] = temporal_resolution_path
                run_details['descriptive_temporal_resolution'] = descriptive_temporal_resolution
            
                #save data to configuration.npz file
                np.savez_compressed('GHOST_configuration.npz', run_details=run_details, parameter_details=parameter_details, network_details=network_details)
    
###------------------------------------------------------------------------------------###
#STAGE T1 : 1. Temporally grid station specific data to desired temporal resolution by year/month
#           2. Outputs station specific .npz files to respective /temporal_resolution/component/temporary/year/month directories 
###------------------------------------------------------------------------------------###
            
                if ('T1' in stages_to_process) & (temporal_process_code == 0):
                    temporal_process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'T1', t0)  
                
###------------------------------------------------------------------------------------###
#STAGE T2 : #1. Merge temporary station specific files to 1 NETCDF by year/month 
            #2. Outputs NETCDF (.nc) files by year/month to /temporal_resolution/component/ directory
###------------------------------------------------------------------------------------###

                if ('T2' in stages_to_process) & (temporal_process_code == 0):
                    temporal_process_code = crossnetwork_processing_modules.submit_slurm_from_python(n_processes, 'T2', t0) 

                #At least one temporal resolutions has been processed successfully, so process is successful, adjust process_code accordingly
                if temporal_process_code == 0:
                   process_code = 0 

###------------------------------------------------------------------------------------###
#FINALISE PROCESS : 1. Stop timer 
###------------------------------------------------------------------------------------###

        if process_code == 0:
            #Stop timer
            t1 = time.time()
            total = t1-t0
            print 'PROCESSING OF ALL DESIRED STAGES FOR %s:%s COMPLETE IN %s MINUTES'%(network,parameter,total/60.)
            print ''
    
###------------------------------------------------------------------------------------###
###------------------------------------------------------------------------------------###